<?php
/**
 * WoWRoster.net WoWRoster
 *
 * EventCalendar Update Hook
 *
 * LICENSE: Licensed under the Creative Commons
 *          "Attribution-NonCommercial-ShareAlike 2.5" license
 *
 * @copyright  2002-2007 WoWRoster.net
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5   Creative Commons "Attribution-NonCommercial-ShareAlike 2.5"
 * @version    SVN: $Id: install.def.php 346 2007-12-24 07:01:07Z tuigii $
 * @link       http://www.wowroster.net
 * @package    EventCalendar
*/

if ( !defined('IN_ROSTER') )
{
    exit('Detected invalid access to this file!');
}

/**
 * Addon Update class
 * This MUST be the same name as the addon basename
 */
class EventCalendarUpdate
{
	var $messages = '';		// Update messages
	var $data = array();		// Addon config data automatically pulled from the addon_config table
	var $files = array();	// We need on of them

	/**
	 * Class instantiation
	 * The name of this function MUST be the same name as the class name
	 *
	 * @param array $data	| Addon data
	 * @return recipe
	 */
	function EventCalendarUpdate($data)
	{
		global $roster;

		$this->data = $data;
		$this->files[] = 'guildeventmanager2'; // Register this file for upload
	}

	/**
	 * Resets addon messages
	 */
	function reset_messages()
	{
		$this->messages = '';
	}

 /**
 	* Standalone Update Hook
  *
  * @return bool
  */
 
	function update( )
	{
		global $roster, $update;

		$this->reset_messages();
		
		// Proceed if we have the green light
		// The admin can block uploads if needed.
		if( $this->data['config']['EventCalendar_UpdateTrigger'] == 1)
		{
			// Sanity check - if GEM_Events data is there, use it.
			if( isset($update->uploadData['guildeventmanager2']))
			{
				$output = $this->processGEM($update->uploadData['guildeventmanager2']);
			}
		}
		else
		{
			$output = '<li>'. $roster->locale->act['EventCalendar_NoUpload'] . '</li>\n';
		}
		$this->messages = $output; // Show time
		return true;
	}
	
	function processGEM($eventdata)
	{
		global $roster, $update, $addon;

		$output = "<span class=\"yellow\">Processing <b>GuildEventManager</b> Data</span><br/>\n";
		$output .= "<ul>\n";

		// First, check presence of channels in uploaded file - at least ON, or all of them
	
		$UpdateValidated_1 = false;
		$UpdateValidated_2 = false;
		$foundchannel = array();
	
		// First, look for all GEM channels in all realms for all subscribing characters
		// These are the channels we are looking for
		$wantedchannels = explode(",",$this->data['config']['EventCalendar_GEMChannelNames']);

		$realms = $eventdata['GEM_Events']['realms'];
		$realm = key($realms);
		$val = current($realms);
		reset($realms);

		while(list ($realm, $val) = each ($realms))
		{
			$my_names = $eventdata['GEM_Events']['realms'][$realm]['my_names'];
			foreach($my_names as $my_name)
			{			
				foreach($my_name['channels'] as $channel)
				{
					if(in_array($channel['name'], $wantedchannels))
					{
						if(!in_array($channel['name'], $foundchannel))
						{
							$foundchannel[] = $channel['name'];
						}
					} 
				}
			}
		}
		
		$nchannelsfound = count($foundchannel);

		if($nchannelsfound > 0)
		{
			// We want one channel, then ok, we have it
			if(count($wantedchannels) == 1)
			{
				$output .= sprintf("<li>%s :[<span class=\"green\"><b>%s</b></span>]</li>",
					$roster->locale->act['EventCalendar_AcceptOneChannel'],$foundchannel[0]);
				$UpdateValidated_1 = true;
			}
			else
			{
				// We found more channels in the uploaded file, and more then one is wanted
				// Do we accept at least on of them, or should the found channels be equal to the wanted channels
				// All this hassle because we're gona wipe the database on update - better have the info for all of them on rebuild
				// Say, our guild uses 2 channels, chan1 and chan2 - these are the accepted/wanted ones
				// Our player has these 2 channels and we need them = ok - ((need all) && (wanted == found)) = true
				// Our player has only 1 channel 'chanl' and (need all) is false, then also ok
				// Note that in this case no more info about 'chan2' will be present
				if( ( ($this->data['config']['EventCalendar_MatchAllChannels'] == 1) && (count($wantedchannels) == $nchannelsfound))
				   || ($this->data['config']['EventCalendar_MatchAllChannels'] == 0) )
				{

					$outtmp = sprintf("<li>%s</li>", $roster->locale->act['EventCalendar_AcceptMoreChannel']);
					$output .= sprintf($outtmp, $nchannelsfound, "[<span class=\"green\"><b>".$this->ListChannels($foundchannel)."</b></span>]</li>");
					$UpdateValidated_1 = true;
				}
				else
				{
					// Give a nice error message, Flash in the future
					$reason = sprintf("%s", $roster->locale->act['EventCalendar_Reason1']);
					$reason = sprintf($reason, "[<span class=\"red\"><b>".$this->ListChannels($wantedchannels)."</b></span>]");
				}
			}
		}
		else
		{
			// Another "why it went wrong" message
		
			$reason = sprintf("%s", $roster->locale->act['EventCalendar_Reason2']);
			$reason = sprintf($reason, "[<span class=\"red\"><b>".$this->ListChannels($wantedchannels)."</b></span>]");
		}

		// Next, check if you have at least one player with a Guild Title (MG, Off, etc), that has the permitted Guild Rank to update
		// This implies that the name found must be member of the roster member list (= Guild member)
		// This also implies that a high rank member from Guild X can't update our GEM addon, even if he has the right channels in his update file
		// Still to do : make update possible for a selected group of members, by name (as the original 1.7.3 addon did).
		// Do this if first check ($UpdateValidated_1) was passed ok

		$foundrank = 10;
		if($UpdateValidated_1)
		{
			$realms = $eventdata['GEM_Events']['realms'];
			$realm = key($realms);
			$val = current($realms);
			reset($realms);

			while(list($realm, $val) = each($realms))
			{
				$my_names = $eventdata['GEM_Events']['realms'][$realm]['my_names'];
				$my_name = key($my_names);
				$val2 = current($my_names);
				reset($my_names);
			
				while(list($my_name, $val2) = each($my_names))
				{
					$thismemberrank = $this->GetRosterMemberRank($my_name);
					if ($thismemberrank < $foundrank)
					{
						$foundrank = $thismemberrank;
					}
					if($thismemberrank <= $this->data['config']['EventCalendar_PlayerLevelToUpdate'])
					{
						// Found a member with a rank lower or equal then needed - 0 means MG
						$UpdateValidated_2 = true;
						$log_name = $my_name;
					}
				}
			}
			if(!$UpdateValidated_2)
			{
				if($foundrank == 10)
				{
					$reason = $roster->locale->act['EventCalendar_Reason4'];
				}
				else
				{
					$reason = sprintf($roster->locale->act['EventCalendar_Reason3'],
						"'<span class=\"red\"><b>".$foundrank."</b></span>'",
						"'<span class=\"red\"><b>".$this->data['config']['EventCalendar_PlayerLevelToUpdate']."</b></span>'",
						"'<span class=\"green\"<b>0</b></span>'" );
				}
			}
		}
		
		if($UpdateValidated_1 && $UpdateValidated_2)
		{
			// Do the actual update for all realms, all events
			// First, wipe the actual data base - we keep the logs	
			$this->ClearDB();
			
			$realms = $eventdata['GEM_Events']['realms'];
			$realm = key($realms);
			$val = current($realms);
			reset($realms);
			
			$Events_parsed = 0;	
			// Parse all realms
			while(list ($realm, $val) = each ($realms))
			{
				//Put each event in array and then process them
				$events = $eventdata['GEM_Events']['realms'][$realm]['events'];
				foreach(array_keys($events) as $event)
				{
					
					$channel = $events[$event]['channel'];
					if(in_array($channel,$wantedchannels))
					{
						$output .= $this->updateGemEvent($events[$event]);
						$Events_parsed = $Events_parsed + 1;
					}
				}
			}
			
			// Log if asked
			if($this->data['config']['EventCalendar_LogEventsUpload'] == 1)
			{
				// Add event log to db->event table
				$update->reset_values();
				$update->add_value('name', $log_name);
				$update->add_value('date', date("Y-m-d G:i:s"));
				$update->add_value('number', $Events_parsed);
	
				$query_log = "INSERT INTO `".$roster->db->table('event_log',$this->data['basename'])."` SET " . $update->assignstr. ";";
				$result = $roster->db->query($query_log);
			}
		}
		else
		{
			$outtmp = sprintf("<li>%s</li>", $roster->locale->act['EventCalendar_AcceptRejected']);
			$output .= sprintf($outtmp, $this->files[0], $reason);
		}
		
		$output .= "</ul>\n"; 

		return $output;
	}

	function updateGemEvent($eventdata)
	{
		global $roster, $update;
	
		$date = $eventdata['ev_date'];
		$type = $eventdata['ev_place'];
		$note = $eventdata['ev_comment'];
		$leader = $eventdata['leader'];
		$minLevel = $eventdata['min_lvl'];
		$maxLevel = $eventdata['max_lvl'];
		$maxCount = $eventdata['max_count'];
		
		$date = date("Y-m-d G:i:s",$date);
	
		// Add eventinfo to db->event table
		$update->reset_values();
		$update->add_value('date', $date);
		$update->add_value('type', $type);
		$update->add_value('note', $note);
		$update->add_value('leader', $leader);
		$update->add_value('minLevel', $minLevel);
		$update->add_value('maxLevel', $maxLevel);
		$update->add_value('maxCount', $maxCount);
		
		$query_addevent = "INSERT INTO `".$roster->db->table('events',$this->data['basename'])."` SET " . $update->assignstr. ";";
		$result = $roster->db->query($query_addevent);

		// Get eventid we just created
		$query_eventid = "SELECT eventid FROM `".$roster->db->table('events',$this->data['basename'])."` WHERE date = '$date' AND leader = '$leader'" ;
		$eventid = $roster->db->query_first($query_eventid);

		$this->WritePlayersList($eventid, $eventdata['titulars'], "Y"); // Players comfirmed
		$this->WritePlayersList($eventid, $eventdata['substitutes'], "S"); // Players Standby
		$this->WritePlayersList($eventid, $eventdata['replacements'], "R"); // Players Replacement

		# Record for limits table
		while (list($class, $limit) = each($eventdata['classes']))
		{
			$class = substr($class, 0, 1).strtolower(substr($class,1));
			$min = ($limit['min'] == -1?0:$limit['min']);
			$max = ($limit['max'] == -1?0:$limit['max']);
			
			# add new limits
			$update->reset_values();
			$update->add_value('eventid', $eventid);
			$update->add_value('class', $class);
			$update->add_value('min', $min);
			$update->add_value('max', $max);
		
			$query_addlimits = "INSERT INTO `".$roster->db->table('event_limits',$this->data['basename'])."` SET " . $update->assignstr;
				
			if($min != -1 && $max != -1)
			{
				$roster->db->query($query_addlimits);
			}
		}
	
		// Tell the world that we recorded an event.
		$output = '<li>Inserting ' . $type .' ('. $date .'), organised by '.$leader.'</li>' ;
		return $output;
	}

	// helper functions for the main update routine

	function WritePlayersList($eventid, $titulars, $typestatus)
	{
		global $roster, $update;

		while (list($id,$player) = each($titulars))
		{
			# subscriber info
			$name = $player['name'];

			if(!isset($player['place']))
			{
				 // Found that 'place' (P1, P2, ... isn't always present !?!
			}
			$place = $player['place'];
			$status = $typestatus;
			$note = "";
			
			# member info
			$guild = $player['guild'];
			$class = $player['class'];
			$class = substr($class, 0, 1).strtolower(substr($class,1)); // Make first letter capital, lower the rest i.e. HUNTER => Hunter
			$level = $player['level'];
			
			// Add to subsriber table
			$update->reset_values();
			$update->add_value('eventid', $eventid);
			$update->add_value('name', $name);
			$update->add_value('place', $place);
			$update->add_value('status', $status);
			$update->add_value('note', $note);
			
			$query_addsubscriber = "INSERT INTO `".$roster->db->table('event_subscribers',$this->data['basename'])."` SET " . $update->assignstr;
			$roster->db->query($query_addsubscriber);
			
			// add players if they don't exist
			$query_player = "SELECT name FROM `".$roster->db->table('event_members',$this->data['basename'])."` WHERE name = '$name'";
			$result_player = $roster->db->query($query_player);
			$memberInfo = $roster->db->fetch( $result_player );
			$roster->db->free_result($result_player);
				
			if ($memberInfo)
			{
				# update player if needed
				$subquery = "SELECT name FROM `".$roster->db->table('event_members',$this->data['basename'])."` WHERE name = '$name' and class = '$class' and level = '$level' and guild = '".addslashes($guild)."';";
				$subresult = $roster->db->query($subquery);
				$levelinfo = $roster->db->fetch( $subresult );
				$roster->db->free_result($subresult);
			
				if($levelinfo)
				{ }
				else
				{
					$update->reset_values();
					$update->add_value('level', $level);
					if($guild != 'N/D')
					{
						$update->add_value('guild', $guild);
					}
			
					$query_updateplayer = "UPDATE `".$roster->db->table('event_members',$this->data['basename'])."` SET " . $update->assignstr . " WHERE name = '$name'";
					$roster->db->query($query_updateplayer);
				}
			}
			else
			{
				# add new player
				$update->reset_values();
				$update->add_value('name', $name);
				$update->add_value('guild', $guild);
				$update->add_value('class', $class);
				$update->add_value('level', $level);
				
				$query_addplayer = "INSERT INTO `".$roster->db->table('event_members',$this->data['basename'])."` SET " . $update->assignstr;
				$roster->db->query($query_addplayer);
			}		
		} 
	}
	
	// Gives out a comma space separated list
	function ListChannels($listedchannel)
	{
		$first = true;
		$output = '';
		foreach($listedchannel as $channel)
		{
			if($first)
			{
				$output .= $channel;
				$first = false;
			}
			else
			{
				$output .= ', '.$channel; 
			}
		}
		return $output;
	}
	
	// Get member info => his rank. 10 means : no member (ranks go from 0 to 9)
	function GetRosterMemberRank($name)
	{
		global $roster;

		// Check if members info exists [read : is member of guild] and return guild_rank.
		$query = 'SELECT guild_rank FROM '.$roster->db->table('members').' WHERE name= "'.$name.'"';
		$result = $roster->db->query($query);
		$rank = $roster->db->fetch($result);
		$roster->db->free_result($result);
		
		if($rank[0] != '')
		{
			return $rank[0];
		}
		else
		{
			return 10;
		}
	}

	// Wipe our precious data - we're about to update anyway
	function ClearDB()
	{
		global $roster;

		$truncate_events = "TRUNCATE `" . $roster->db->table('events',$this->data['basename']) . "`";
		$truncate_event_limits = "TRUNCATE `" . $roster->db->table('event_limits',$this->data['basename']) . "`";
		$truncate_event_subscribers = "TRUNCATE `" . $roster->db->table('event_subscribers',$this->data['basename']) . "`";
		$truncate_event_members = "TRUNCATE `" . $roster->db->table('event_members',$this->data['basename']) . "`";
		
		$roster->db->query($truncate_events) or die_quietly( $roster->db->error(),'Database Error',basename(__FILE__),__LINE__,$truncate_events);
		$roster->db->query($truncate_event_limits) or die_quietly( $roster->db->error(),'Database Error',basename(__FILE__),__LINE__,$truncate_event_limits);
		$roster->db->query($truncate_event_subscribers) or die_quietly( $roster->db->error(),'Database Error',basename(__FILE__),__LINE__,$truncate_event_subscribers);
		$roster->db->query($truncate_event_members) or die_quietly( $roster->db->error(),'Database Error',basename(__FILE__),__LINE__,$truncate_event_members);
		
		return "<li>" . $roster->locale->act['EventCalendar_ClearDB'] ."</li>";
	}
}
?>